/*    
    Copyright (C) 2012 http://software-talk.org/ (developer@software-talk.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mill.Controller;

import java.util.logging.Level;
import java.util.logging.Logger;
import de.uni.kiel.proki.ws1112.muehle.interfaces.GameException;
import de.uni.kiel.proki.ws1112.muehle.interfaces.Move;
import java.awt.event.MouseEvent;
import de.uni.kiel.proki.ws1112.muehle.interfaces.ModelInterface;
import de.uni.kiel.proki.ws1112.muehle.interfaces.PlayerInterface;
import de.uni.kiel.proki.ws1112.muehle.interfaces.Token;
import mill.Model.AbstractPlayer;
import mill.Model.Buffer;
import mill.Model.MyMove;
import mill.view.GameLabel;

/**
 * This class reads input from a human player.
 * @version 1.0.2
 */
public class HumanInputController extends AbstractPlayer implements PlayerInterface {

    private Buffer<MouseEvent> buff; // stores one user mouse event
    private boolean playsLeftMouseButton;

    /**
     * constructs new HumanInputController.
     * @param buff buff
     * @param playsLeftMouseButton playsLeftMouseButton
     * @param name name
     * @param numberOfStones numberOfStones to place
     */
    public HumanInputController(Buffer<MouseEvent> buff, boolean playsLeftMouseButton,
            String name, int numberOfStones) {
        super(name, numberOfStones);
        this.buff = buff;
        this.playsLeftMouseButton = playsLeftMouseButton;
    }
    private MyMove nextMove;

    @Override
    public Move getNextMove(ModelInterface model) {
        return getNextMove(model, 0);
    }

    /**
     * returns a move of a stone on the board.
     * <p>
     * @param model model
     * @param turnCounter turnCounter
     * @return move of a stone on the board
     */
    @Override
    public Move getNextMove(ModelInterface model, int turnCounter) {
        nextMove = new MyMove();
        getNextMoveSource(model);
        getNextMoveDest();
        return nextMove;
    }

    /**
     * adds a source to the <code>nextMove</code> from player input.
     * 
     * @param model model
     */
    private void getNextMoveSource(ModelInterface model) {
        while (true) {
            try {
                MouseEvent event = buff.read();
                GameLabel temp = (GameLabel) event.getComponent();
                nextMove.setSource(temp.getColumn(), temp.getRow());
                if (model.getFieldStatus(nextMove.getSource().x, nextMove.getSource().y) == super.getColor()) {

                    if (event.getButton() == MouseEvent.BUTTON1
                            && playsLeftMouseButton) {

                        // TODO VIEW ich finde, dass die umrandung nach dem waehlen der source das ganze sehr viel uebersichtlicher macht (die farbe kannste gerne aendern^^).
                        // ich haette auch gerne sowas noch fuer die ki (fuer dest und src) und fuer dest hier.
                        // es reicht wenn das nur kurz aufblinkt oder so.
                        temp.setIsSelected(true); // marks label with border
                        temp.updateUI();
                        break;

                    } else if (event.getButton() == MouseEvent.BUTTON3
                            && !playsLeftMouseButton) {
                        temp.setIsSelected(true); // marks label with border
                        temp.updateUI();
                        break;
                    }
                }
            } catch (GameException ex) {
                Logger.getLogger(HumanInputController.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * adds a dest to the <code>nextMove</code> from player input.
     * <p>
     **/
    private void getNextMoveDest() {
        while (true) {
            MouseEvent event = buff.read();


            if (event.getButton() == 1
                    && playsLeftMouseButton) {

                GameLabel temp = (GameLabel) event.getComponent();
                nextMove.setDest(temp.getColumn(), temp.getRow());
//                temp.setIsSelected(true); // marks label with border
//                temp.updateUI();
                break;

            } else if (event.getButton() == 3
                    && !playsLeftMouseButton) {
                GameLabel temp = (GameLabel) event.getComponent();
                nextMove.setDest(temp.getColumn(), temp.getRow());
//                temp.setIsSelected(true); // marks label with border
//                temp.updateUI();
                break;
            }
        }
    }

    @Override
    public Move getNextSetMove(ModelInterface model) {
        return getNextSetMove(model, 0);
    }

    /**
     * returns a position to place a stone on the board.
     * <p>
     * The position will be saved in the destination of the move.
     * <p>
     * @param model model
     * @param turnCounter turnCounter
     * @return position (in destination of the move)
     */
    @Override
    public Move getNextSetMove(ModelInterface model, int turnCounter) {
        while (true) {
            MouseEvent event = buff.read();


            if (event.getButton() == 1
                    && playsLeftMouseButton) {
                GameLabel temp = (GameLabel) event.getComponent();
                MyMove move = new MyMove();
                move.setDest(temp.getColumn(), temp.getRow());
                return move;

            } else if (event.getButton() == 3
                    && !playsLeftMouseButton) {
                GameLabel temp = (GameLabel) event.getComponent();
                MyMove move = new MyMove();
                move.setDest(temp.getColumn(), temp.getRow());
                return move;
            }
        }
    }

    /**
     * returns a position to remove a stone on the board.
     * <p>
     * The position will be saved in the destination of the move.
     * <p>
     * WARNING: null will be returned if the <code>MouseEvent</code> is null.
     * In this case the thread is interupted by the gui to restart game.
     * <p>
     * @param model model
     * @return position (in destination of the move)
     */
    @Override
    public Move getRemoveMove(ModelInterface model) {
        return getRemoveMove(model, 0);
    }

    @Override
    public Move getRemoveMove(ModelInterface model, int time) {
        while (true) {
            MouseEvent event = buff.read();



            if (event.getButton() == 1
                    && playsLeftMouseButton) {
                GameLabel temp = (GameLabel) event.getComponent();
                MyMove move = new MyMove();
                move.setDest(temp.getColumn(), temp.getRow());
                return move;

            } else if (event.getButton() == 3
                    && !playsLeftMouseButton) {
                GameLabel temp = (GameLabel) event.getComponent();
                MyMove move = new MyMove();
                move.setDest(temp.getColumn(), temp.getRow());
                return move;
            }
        }
    }
}
