/*    
    Copyright (C) 2012 http://software-talk.org/ (developer@software-talk.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mill.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.LinkedList;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import mill.Controller.MainController;
import de.uni.kiel.proki.ws1112.muehle.interfaces.GameException;
import de.uni.kiel.proki.ws1112.muehle.interfaces.ModelInterface;
import de.uni.kiel.proki.ws1112.muehle.interfaces.Token;
import mill.Model.OptionObject;

/**
 * this class provides the gui for the game "Mill".
 * @version 1.0.5
 */
public class GameGui extends javax.swing.JFrame {

    /**
     * Creates new form GameGui.
     *
     * @param ctrl the game controller
     */
    public GameGui(MainController ctrl) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            System.out.println("could not set systems look and feel, using java default.");
        }
        this.ctrl = ctrl;
        this.options = ctrl.getOptions();
        this.gameHeight = ctrl.getHeight();
        this.gameWidth = ctrl.getWidth();
        initComponents();
        init();
        labels = new GameLabel[gameHeight][gameWidth];
        makeLabels();
        messages = new LinkedList<String>();
        repaint();
        //closing operation:
        this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent we) {
                saveOptions();
                // TODO dont use exit()
                System.exit(0);
            }
        ;
    }
    );
    }
    private MainPanel mainPanel;
    private GameLabel[][] labels;
    private MainController ctrl;
    private int gameHeight;
    private int gameWidth;
    private JLayeredPane pane;
    private JPanel backgroundPanel;
    private JTextArea textOutput;
    private LinkedList<String> messages;
    private OptionObject options;

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        playerButtonGroup = new javax.swing.ButtonGroup();
        menu = new javax.swing.JMenuBar();
        gameMenu = new javax.swing.JMenu();
        startButton = new javax.swing.JMenuItem();
        exitButton = new javax.swing.JMenuItem();
        optionMenu = new javax.swing.JMenu();
        switchDiagonalRuleButton = new javax.swing.JCheckBoxMenuItem();
        moreOptionsButton = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        humanVsHumanButton = new javax.swing.JCheckBoxMenuItem();
        humanVsComButton = new javax.swing.JCheckBoxMenuItem();
        comVsHumanButton = new javax.swing.JCheckBoxMenuItem();

        setTitle("Mill X");
        setMinimumSize(new java.awt.Dimension(400, 400));
        setName("mainFrame"); // NOI18N

        gameMenu.setText("Game");

        startButton.setText("Start");
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });
        gameMenu.add(startButton);

        exitButton.setText("Exit");
        exitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitButtonActionPerformed(evt);
            }
        });
        gameMenu.add(exitButton);

        menu.add(gameMenu);

        optionMenu.setText("Options");

        switchDiagonalRuleButton.setSelected(true);
        switchDiagonalRuleButton.setText("Play with diagonal lines");
        switchDiagonalRuleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                switchDiagonalRuleButtonActionPerformed(evt);
            }
        });
        optionMenu.add(switchDiagonalRuleButton);

        moreOptionsButton.setText("More options");
        moreOptionsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moreOptionsButtonActionPerformed(evt);
            }
        });
        optionMenu.add(moreOptionsButton);
        optionMenu.add(jSeparator1);

        playerButtonGroup.add(humanVsHumanButton);
        humanVsHumanButton.setSelected(true);
        humanVsHumanButton.setText("Human vs. Human");
        humanVsHumanButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                humanVsHumanButtonActionPerformed(evt);
            }
        });
        optionMenu.add(humanVsHumanButton);

        playerButtonGroup.add(humanVsComButton);
        humanVsComButton.setText("Human vs. computer");
        humanVsComButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                humanVsComButtonActionPerformed(evt);
            }
        });
        optionMenu.add(humanVsComButton);

        playerButtonGroup.add(comVsHumanButton);
        comVsHumanButton.setText("Computer vs. human");
        comVsHumanButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comVsHumanButtonActionPerformed(evt);
            }
        });
        optionMenu.add(comVsHumanButton);

        menu.add(optionMenu);

        setJMenuBar(menu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 287, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButtonActionPerformed
        ctrl.startGame();
    }//GEN-LAST:event_startButtonActionPerformed

private void exitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitButtonActionPerformed
    saveOptions();
    // TODO dont use exit()
    System.exit(0);
}//GEN-LAST:event_exitButtonActionPerformed

    private void switchDiagonalRuleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_switchDiagonalRuleButtonActionPerformed
        options.setPlayWithDiagonalLines(!options.isPlayWithDiagonalLines());
    }//GEN-LAST:event_switchDiagonalRuleButtonActionPerformed

    private void humanVsHumanButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_humanVsHumanButtonActionPerformed
        options.setPlayerOneAsComputer(false);
        options.setPlayerTwoAsComputer(false);
    }//GEN-LAST:event_humanVsHumanButtonActionPerformed

    private void humanVsComButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_humanVsComButtonActionPerformed
        options.setPlayerOneAsComputer(false);
        options.setPlayerTwoAsComputer(true);
    }//GEN-LAST:event_humanVsComButtonActionPerformed

    private void comVsHumanButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comVsHumanButtonActionPerformed
        options.setPlayerOneAsComputer(true);
        options.setPlayerTwoAsComputer(false);
    }//GEN-LAST:event_comVsHumanButtonActionPerformed

    private void moreOptionsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moreOptionsButtonActionPerformed
        createOptionDialog();
        this.setEnabled(false);
    }//GEN-LAST:event_moreOptionsButtonActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBoxMenuItem comVsHumanButton;
    private javax.swing.JMenuItem exitButton;
    private javax.swing.JMenu gameMenu;
    private javax.swing.JCheckBoxMenuItem humanVsComButton;
    private javax.swing.JCheckBoxMenuItem humanVsHumanButton;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JMenuBar menu;
    private javax.swing.JMenuItem moreOptionsButton;
    private javax.swing.JMenu optionMenu;
    private javax.swing.ButtonGroup playerButtonGroup;
    private javax.swing.JMenuItem startButton;
    private javax.swing.JCheckBoxMenuItem switchDiagonalRuleButton;
    // End of variables declaration//GEN-END:variables

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        pane.setBounds(0, 0, getWidth(), getHeight());
        backgroundPanel.setBounds(0, 0, pane.getWidth(), pane.getHeight());
        textOutput.setBounds(0, 0, pane.getWidth(), 50);
        makePanelSquare();
        pane.moveToBack(backgroundPanel);
        pane.moveToFront(mainPanel);
        pane.moveToFront(textOutput);
        setBackgroundColor(options.getBackgroundColor());
        setTextAreaBackgroundColor(options.getTextAreaBackgroundColor());
        setTextColor(options.getTextColor());
    }

    /**
     * updates the complete gui game field and print it to the screen.
     * @param field the field to print
     */
    public void updateGuiGameField(ModelInterface field) {
        for (int i = 0; i < field.getHeight(); i++) {
            for (int j = 0; j < field.getWidth(); j++) {
                try {
                    GameLabel label = getLabelAt(j, i);
                    Token fieldStatus = field.getFieldStatus(j, i);
                    label.setIsSelected(false);
                    label.updateUI();
                    if (fieldStatus == Token.EMPTY) {
                        label.clear();
                    } else {
                        label.clear();
                        label.setCircle();
                        Color setColor;
                        if (fieldStatus == Token.BLACK) {
                            setColor = Color.BLACK;
                        } else {
                            setColor = Color.WHITE;
                        }
                        label.setColor(setColor);
                    }
                } catch (GameException ex) {
                }
            }
        }
        
        repaint();
    }

    public void animateGameFieldChanges(ModelInterface field) {
        for (int i = 0; i < field.getHeight(); i++) {
            for (int j = 0; j < field.getWidth(); j++) {
                try {
                    GameLabel currentLabel = getLabelAt(j, i);
                    Token currentFieldStatus = field.getFieldStatus(j, i);
                    if (!currentLabel.isCircle() && currentFieldStatus.equals(Token.EMPTY)) {
                        // do nothing!
                    } else if (!currentLabel.isCircle() && !currentFieldStatus.equals(Token.EMPTY)) {
//                            currentLabel.setCircle();
//                            Color setColor;
//                            if (currentFieldStatus == Token.BLACK) {
//                                setColor = Color.BLACK;
//                            } else {
//                                setColor = Color.WHITE;
//                            }
//                            currentLabel.setColor(setColor);
                        currentLabel.setAsTarget(true);
                        break;
                        // currentLabel.updateUI();
                    } else if (currentLabel.isCircle() && currentFieldStatus.equals(Token.EMPTY)) {
                        //currentLabel.clear();
                        currentLabel.setAsSource(true);
                        break;
                        //currentLabel.updateUI();
                    }

                } catch (GameException ex) {
                }
            }
        }
        repaint();
    }

    /**
     * says the controller to save the options.
     */
    private void saveOptions() {
        ctrl.saveOptions();
    }

    /**
     * makes a new main panel where the game would be paint.
     * Instance a <code>JLayeredPane</code> and a pnale for the
     * backgroundcolor. Also makes a <code>JTextArea</code> for the text
     * output. Additionally it select the right buttons depending on 
     * the saving options.
     */
    private void init() {
        pane = new JLayeredPane();
        this.add(pane);
        pane.setBounds(0, 0, getWidth(), getHeight());
        this.backgroundPanel = new JPanel();
        pane.add(backgroundPanel);
        mainPanel = new MainPanel();
        mainPanel.setLayout(new GridLayout(gameWidth, gameHeight));
        textOutput = new JTextArea(3, 30);
        textOutput.setEditable(false);
        textOutput.setFont(new Font("Verdana", Font.PLAIN, 10));
        textOutput.append("welcome.");
        pane.add(textOutput);
        pane.add(mainPanel);
        repaint();

        boolean isP1Com = options.isPlayerOneComputer();
        boolean isP2Com = options.isPlayerTwoComputer();
        if (!isP1Com && !isP2Com) {
            selectHumanVsHumanButton();
        } else if (!isP1Com && isP2Com) {
            selectHumanVsComButton();
        } else if (isP1Com && !isP2Com) {
            selectComVsHumanButton();
        }
        selectSwitchDiagonalButton(options.isPlayWithDiagonalLines());
    }

    /**
     * make gamefieldwidth * gamefieldheight labels and
     * adds a mouselistener if on the label could be place something.
     */
    private void makeLabels() {
        for (int row = 0; row < gameWidth; row++) {
            for (int column = 0; column < gameHeight; column++) {
                GameLabel newLabel = new GameLabel(column, row);
                if (ctrl.getBelongsToGame(column, row)) {
                    newLabel.addMouseListener(new GuiMouseListener());
                }
                labels[column][row] = newLabel;
                mainPanel.add(newLabel);
            }
        }
        makePanelSquare();
    }

    /**
     * returns the <code>GameLabel</code> at the
     * specific position.
     * 
     * @param row the row
     * @param column the column
     * @return the <code>GameLabel</code> at the position
     */
    public GameLabel getLabelAt(int column, int row) {
        return labels[column][row];
    }

    /**
     * call the <code>clear()</code> method of all
     * labels.
     */
    public void resetLabels() {
        for (int row = 0; row < gameWidth; row++) {
            for (int column = 0; column < gameHeight; column++) {
                labels[column][row].clear();
            }
        }
    }

    /**
     * this method provides that the <code>mainPanel</code> is square, alse
     * when the window is resized.
     */
    private void makePanelSquare() {
        int pHeight = pane.getSize().height - textOutput.getHeight() - 15;
        int pWidth = pane.getSize().width;
        int size = 0;
        if (pHeight >= pWidth) {
            size = pWidth - 40;
        } else {
            size = pHeight - 40;
        }
        mainPanel.setBounds(0, textOutput.getHeight(), size, size);
    }

    /**
     * says the main panel to draw the diagonal lines.
     * <p>
     * WARNING: <code>repaint()</code> must be called to draw this
     * correctly.
     * <p>
     * @param playWithDiagonalLines to set diagonal lines on or off
     */
    public void setDiagonalLines(boolean playWithDiagonalLines) {
        mainPanel.setPlayWithDiagonalLines(playWithDiagonalLines);
    }

    /**
     * prints the message to the <code>textOutput</code> label in the gui.
     * Up to three messages will be print at one time.
     * @param msg the message to print
     */
    public void sendMessege(String msg) {
        messages.addLast(msg);
        if (messages.size() > 3) {
            messages.removeFirst();
        }
        String output = "";
        for (String string : messages) {
            output += string + '\n';
        }
        textOutput.setText(output);
    }

    /**
     * set the <code>humanVsHumanButton</code> selected,
     * and the other player buttons not selected.
     */
    public void selectHumanVsHumanButton() {
        playerButtonGroup.setSelected(humanVsHumanButton.getModel(), true);
    }

    /**
     * set the <code>comVsHumanButton</code> selected,
     * and the other player buttons not selected.
     */
    public void selectComVsHumanButton() {
        playerButtonGroup.setSelected(comVsHumanButton.getModel(), true);
    }

    /**
     * set the <code>humanVsComButton</code> selected,
     * and the other player buttons not selected.
     */
    public void selectHumanVsComButton() {
        playerButtonGroup.setSelected(humanVsComButton.getModel(), true);
    }

    /**
     * select or deselect the button which switchses the rule
     * for diagonal gaming on and off.
     * @param isSelected true if button is selected, else false.
     */
    public void selectSwitchDiagonalButton(boolean isSelected) {
        switchDiagonalRuleButton.setSelected(isSelected);
    }

    /**
     * sets the backgroundcolor of the game.
     * @param bg the backgroundcolor
     */
    public void setBackgroundColor(Color bg) {
        mainPanel.setBackground(bg);
        backgroundPanel.setBackground(bg);
    }

    /**
     * sets the backgroundcolor of the text area.
     * @param bg the backgroundcolor
     */
    public void setTextAreaBackgroundColor(Color bg) {
        textOutput.setBackground(bg);
    }

    /**
     * sets the color of the text.
     * @param fg the text color
     */
    public void setTextColor(Color fg) {
        textOutput.setForeground(fg);
    }

    private void createOptionDialog() {
        OptionFrame optionFrame = new OptionFrame(ctrl);
        optionFrame.setBounds(this.getX() + 50, this.getY() + 100, 300, 220);
        optionFrame.setVisible(true);
    }
    
  

    /**
     * the mouselistener for the gui to provide clickable 
     * labels, which could draw something. Writes his event 
     * in an buffer for the backend. The label will be has a
     * border if the mouse comes over. The border disappearce if
     * the mouse go outside the label.
     */
    private class GuiMouseListener implements MouseListener {

        @Override
        public void mouseClicked(MouseEvent e) {
        }

        @Override
        public void mousePressed(MouseEvent e) {
            ctrl.processInput(e);
        }

        @Override
        public void mouseReleased(MouseEvent e) {
        }

        @Override
        public void mouseEntered(MouseEvent e) {
            Object source = e.getSource();
            if (source instanceof GameLabel) {
                GameLabel clicked = (GameLabel) source;
                //if (!clicked.isIsSelected()) 
                clicked.setMouseInside(true);
                clicked.updateUI();
                //    clicked.setBorder(BorderFactory.createLineBorder(Color.blue, 2));
                //}
            }
        }

        @Override
        public void mouseExited(MouseEvent e) {
            Object source = e.getSource();
            if (source instanceof GameLabel) {
                GameLabel clicked = (GameLabel) source;
                clicked.setMouseInside(false);
                clicked.updateUI();
                // clicked.setBorder(null);
            }
        }
    }
}
