/*    
    Copyright (C) 2012 http://software-talk.org/ (developer@software-talk.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mill.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.border.Border;
import mill.Model.OptionObject;

/**
 * This class provides an extended JLabel to draw a circle with 
 * a specific color inside. It also saves the real position of the gamefield,
 * which the label represent.
 * @version 1.0.3
 */
public class GameLabel extends JLabel {

    private static final Border SELECTED_BORDER = BorderFactory.createLineBorder(Color.CYAN, 3);
    private static final Border MOUSE_INSIDE_BORDER = BorderFactory.createLineBorder(Color.blue, 2);
    private static final Border SOURCE_BORDER = BorderFactory.createLineBorder(Color.MAGENTA, 3);
    private static final Border TARGET_BORDER = BorderFactory.createLineBorder(Color.ORANGE, 3);

    /* settings for border */
    private boolean isSelected = false;
    private boolean mouseInside = false;
    private boolean changeBorder = false;
    private boolean isTarget = false;
    private boolean isSource = false;
    private boolean isCircle; // checks if a circle should be draw
    private int column;
    private int row;
    private Color color;

    /**
     * makes a new panel and save his position of the gamefield which the
     * label represent.
     * @param row the row of the gamefield
     * @param column the line of the gamefield
     */
    public GameLabel(int column, int row) {
        super();
        this.isCircle = false;
        this.color = Color.BLACK;
        this.column = column;
        this.row = row;
    }

    public boolean isMouseInside() {
        return mouseInside;
    }

    public void setMouseInside(boolean mouseInside) {
        this.mouseInside = mouseInside;
        changeBorder = true;
    }

    /**
     * returns whether or not this label ist currently a target for a move or something.
     * <p>
     * if a label is a targte, the same color how will be paint, as the label is selected,
     * but it will also has a cross inside with the same color.
     * @return  true if the label is a target.
     */
    public boolean isTarget() {
        return isTarget;
    }

    /**
     * sets if the label should be a target or not.
     * <p>
     * if a label is a target, the same color will be paint as the label is selected,
     * but it will also has a cross inside with the same color.
     * **/
    public void setAsTarget(boolean isTarget) {
        this.isTarget = isTarget;
    }
    
    public void setAsSource(boolean isSource){
        this.isSource = isSource;
    }

    /**
     * returns whether or not this label is currently selected.
     * <p>
     * If a label is selected, a border will be drawn around it.
     * <p>
     * <code>IsSelected</code> will be set to false on the next call of <code>clear()</code>.
     *
     * @return
     */
    public boolean isIsSelected() {
        return isSelected;
    }

    /**
     * sets whether or not this label is currently selected.
     * <p>
     * If a label is selected, a border will be drawn around it.
     * <p>
     * <code>IsSelected</code> will be set to false on the next call of <code>clear()</code>.
     *
     * @return
     */
    public void setIsSelected(boolean isSelected) {
        if (isCircle) {
            this.isSelected = isSelected;
            changeBorder = true;
        }
    }

    /**
     * sets the panel to show a circle inside it when <code>draw</code> is called.
     */
    public void setCircle() {
        this.isCircle = true;
    }
    /**
     * checks if the label shows a circle inside.
     * @return true if a circle is painted, else false
     */
    public boolean isCircle() {
        return isCircle;
    }
    

    /**
     * clear the content of the panel and
     * set the boolean flags for <code>isCircle</code> to false.
     */
    public void clear() {
        this.isCircle = false;
        isSelected = false;
        changeBorder = true;
        isTarget = false;
        isSource = false;
        this.setOpaque(false);

    }

    @Override
    public void paintComponent(Graphics g) {
        try {
            super.paintComponent(g);
        } catch (Exception e) {
            // TODO FIXME exception fange ich hier, weil sie mich nervt. aber schoen ist das nicht.
            // try/catch entfernen und gucken woran das liegt das die geworfen wird
            /*
            Exception in thread "AWT-EventQueue-0" java.lang.NullPointerException
            at javax.swing.plaf.synth.SynthContext.getPainter(SynthContext.java:181)
            at javax.swing.plaf.synth.SynthLabelUI.update(SynthLabelUI.java:143)
            at javax.swing.JComponent.paintComponent(JComponent.java:752)
            at mill.view.GameLabel.paintComponent(GameLabel.java:92)
             * [...]
             */
        }
        int distToEdge = (int) (0.3 * super.getWidth());
        int width = super.getWidth() - 2 * distToEdge;
        int height = super.getHeight() - 2 * distToEdge;
        if (isCircle) {
            if (OptionObject.DEBUG) {
                g.setFont(new Font("Verdana", Font.PLAIN, 12));
                g.setColor(Color.GREEN);
                g.drawString(column + "/" + row, this.getSize().width / 2, 10);

            }
            g.setColor(color);
            g.fillOval(distToEdge, distToEdge, width, height);
        }
        if (changeBorder) {
            if (isSelected) {
                this.setBorder(SELECTED_BORDER);
            } else if (mouseInside) {
                this.setBorder(MOUSE_INSIDE_BORDER);
            } else {
                this.setBorder(null);
            }
            changeBorder = false;
        }
        if(isSource){
            this.setBorder(SOURCE_BORDER);
        }
        
        if(isTarget){
            this.setBorder(TARGET_BORDER);
            int thickness =(int) (this.getWidth() * 0.15);
            int x1 = (this.getWidth() / 2) ;
            int y1 = 0;
            int x2 = x1;
            int y2 = getHeight();
            Graphics2D g2d = (Graphics2D) g;
            // vertical line:
            g2d.setColor(Color.ORANGE);
            g2d.setStroke(new BasicStroke(thickness));
            g2d.drawLine(x1, y1, x2, y2);
            // horizontal line:
            x1 = 0;
            y1 = this.getHeight() / 2;
            x2 = this.getWidth();
            y2 = y1;
            g2d.setColor(Color.ORANGE);
            g2d.setStroke(new BasicStroke(thickness));
            g2d.drawLine(x1, y1, x2, y2);
        }
    }

    /**
     * returns the number of columns in the gamefield, which this label
     * represent.
     *
     * @return the number of the column
     */
    public int getColumn() {
        return column;
    }

    /**
     * returns the number of rows, which this label represent.
     *
     * @return the number of the row
     */
    public int getRow() {
        return row;
    }

    /**
     * sets the color of the circle which could draw.
     *
     * @param color the color of the circle
     */
    public void setColor(Color color) {
        this.color = color;
    }

    /**
     * returns the current color of the circle inside.
     * @return the current circle color.
     */
    public Color getColor() {
        return color;
    }
}
